// Copyright 2013 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

static int volatile val = 2;

int test5740a() {
	return val;
}
