// Copyright 2022 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package p

type myString string

func _[P ~string | ~[]byte | ~[]rune]() {
	_ = P("")
	const s myString = ""
	_ = P(s)
}

func _[P myString]() {
	_ = P("")
}
