// Copyright 2021 The Go Authors. All rights reserved.  Use of this
// source code is governed by a BSD-style license that can be found in
// the LICENSE file.

package a

func FM() func() {
	return func() {
		_ = func() int {
			return 0
		}
	}
}
