// rundir

// Copyright 2009 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// See issue 2241 and issue 1878: dwarf include stack size
// issues in linker.

package ignored
